
-- 1.1.3+ compatibility wrapper [function detouring]
-- allows to run scripts written for the old script interface.
-- (C) False.Genesis

-- Read the comments in _loader.lua about how to use this!


-- IMPORTANT that this gets loaded only once. Exit & re-enter the mod if the file was changed.
if rawget(_G, "__COMPAT_LOADED") then
    errorLog("ERROR: _compat.lua already loaded")
    return
end

-- First, be sure we're using a reasonably new version
-- (sorry it's that harsh)
debugLog("_compat.lua: Version check - will crash now if old version")
entity_animate()

-- some helper functions

local rawset = rawset
local rawget = rawget
local debug = rawget(_G, "debug")

local OLD_SCRIPT_PATH = appendUserDataPath("_mods/" .. FG_LOADER_MOD_NAME .. "/scripts/"):lower()
local NEW_SCRIPT_PATH = "scripts2/"
    
local function detourScriptPath(s)
    if s:lower():startsWith(OLD_SCRIPT_PATH) then
        local d = NEW_SCRIPT_PATH .. s:sub(#OLD_SCRIPT_PATH + 1)
        debugLog("DETOURED " .. s .. " -> " .. d)
        return d
    end
    return s
end

local function looksLikeGlobal(s)
    --[[local A = string.byte("A")
    local Z = string.byte("Z")
    local N0 = string.byte("0")
    local N9 = string.byte("9")
    local u = string.byte("_")
    local c 
    for i = 1, #s do
        c = s:byte(i)
        -- accept any uppercase, number, and _ char
        if not ((c >= A and c <= Z) or c == u or (c >= N0 and c <= N9)) then
            return false
        end
    end
    return true]]
    
    -- probably faster
    return not s:match("[^_%u%d]")
end
    
-- explode(string, seperator, skipEmpty)
function string.explode(p, d, skip)
  local t, ll, l, nw
  
  ll = 0
  t = {}
  if(#p == 1) then return {p} end
    while true do
      l=string.find(p,d,ll,true) -- find the next d in the string
      if l~=nil then -- if "not not" found then..
        nw = string.sub(p,ll,l-1)
        if not skip or #nw > 0 then
            table.insert(t, nw) -- Save it in our array.
        end
        ll=l+1 -- save just after where we found it for searching next time.
      else
        nw = string.sub(p,ll)
        if not skip or #nw > 0 then
            table.insert(t, nw) -- Save what's left in our array.
        end
        break -- Break at end, as it should be, according to the lua manual.
      end
    end
  return t
end

function string.startsWith(String,Start)
   return string.sub(String,1,string.len(Start))==Start
end

function string.endsWith(String,End)
   return End=='' or string.sub(String,-string.len(End))==End
end


local function formatStack(lvl)
    if debug then
        if not lvl then lvl = 1 end
        return debug.traceback("", lvl) or "[No traceback available]"
    end
    return "[No debug library available]"
end

local errorLog_o = errorLog
local function errorLogWrap(s)
    return errorLog_o(s .. "\n" .. formatStack(2))
end
rawset(_G, "errorLog", errorLogWrap)

-- detour file loading.
-- the real scripts are in another directory.
-- when attemping to load something in the original dir,
-- redirect to the other dir that has the real files.

local o_dofile = dofile
rawset(_G, "dofileRaw", o_dofile)

local function dofileWrap(file)
    if file:lower() ~= "scripts/entities/entityinclude.lua" then -- already in
        --debugLog("dofile(" .. file .. ")")
        return o_dofile(detourScriptPath(file))
    end
end
rawset(_G, "dofile", dofileWrap)


-- no longer existing old functions --
-- if called, they would throw an error.
-- old scripts may still use them, so we replace them with no-ops to prevent this.

local function dummy()
end

local function mkwarn(s)
    local err = "Dummy function: " .. s .. "() - no longer present in the current API, fix the script!"
    return function() errorLog(err) end
end

local INTERFACE_FUNCTIONS =
{
    action = true,
    activate = true,
    animationKey = true,
    canShotHit = true,
    castSong = true,
    cookFailure = true,
    damage = true,
    deathNotify = true,
    dieEaten = true,
    dieNormal = true,
    enterState = true,
    entityDied = true,
    exitState = true,
    exitTimer = true,
    getIngredientEffectString = true,
    hitEntity = true,
    hitSurface = true,
    init = true,
    lightFlare = true,
    msg = true,
    postInit = true,
    preUpdate = true,
    shiftWorlds = true,
    shotHitEntity = true,
    song = true,
    songNote = true,
    songNoteDone = true,
    sporesDropped = true,
    update = true,
    useTreasure = true,
    useIngredient = true,
}

-- true: warn if called
-- a function: call that instead
local OVERWRITE_FUNCTIONS =
{
    entity_warpToPathStart = true,
    sendEntityMessage = true,
    entity_setCollideWithAvatar = dummy,
    entity_setTouchDamage = dummy,
    bone_setTouchDamage = dummy,
    entity_setClampOnSwitchDir = dummy,
    entity_addGroupVel = dummy,
    entity_avgVel = dummy,
    entity_fireAtTarget = dummy,
    entity_fireShot = true,
    entity_flipHToAvatar = dummy,
    entity_getBehaviorType = dummy,
    entity_getPositionX = entity_x,
    entity_getPositionY = entity_y,
    entity_isSaying = dummy,
    entity_moveTowardsGroupCenter = dummy,
    entity_moveTowardsGroupHeading = dummy,
    entity_applyRandomForce = entity_addRandomVel,
    entity_say = dummy,
    entity_setAffectedBySpells = dummy,
    entity_setBehaviorType = dummy,
    entity_setNodeGroupActive = dummy,
    entity_setSayPosition = dummy,
    entity_setTouchPush = dummy,
    entity_setPauseInConversation = dummy,
    getAngleBetween = true,
    getAngleBetweenEntities = true,
    getNearestNode = true,
    getNodeFromEntity = true,
    healEntity = true,
    killEntity = true,
    learnSpell = dummy,
    reloadTextures = dummy,
    setGameOver = dummy,
    setGLNearest = dummy,
    setNaijaModel = dummy,
    shot_setBounceType = dummy,
    shot_setLifeTime = dummy,
    showControls = dummy,
    stopCursorGlow = dummy,
    toggleTransitFishRide = dummy,
    getNodeByName = getNode,
    getEntityByName = getEntity,
    entity_flipTo = entity_fhTo,
    bone_getidx = bone_getIndex,
}

for name, ov in pairs(OVERWRITE_FUNCTIONS) do
    if rawget(_G, name) then
        debugLog("WARNING: _compat.lua: function " .. name .. " already exists")
    else
        if ov == true then
            rawset(_G, name, mkwarn(name))
        else
            rawset(_G, name, ov)
        end
    end
end


-- fixup functions that differ in behavior now

local createEntity_o = createEntity
rawset(_G, "createEntity", function(e, name, x, y)
    if type(e) == "string" and #e > 0 then
        return createEntity_o(e, name, x, y)
    end
    return 0
end)



-- hook script creation

local function _warnUndefInstance(tab, key)
    if rawget(_G, "FG_ENABLE_DEV_MODE") then
        errorLog_o("WARNING: script tried to get/call undefined instance variable " .. tostring(key) .. "\n" .. formatStack(2))
        rawset(tab, key, false) -- warn only once, not spam
    end
end

local v_meta = { __index = _warnUndefInstance }

local function _detourInit()
    setmetatable(v, v_meta)
end

-- this function must never raise an error, otherwise the program will crash
local function onCreateScript(tab, sc, functable)
    debugLog("=== Creating script instance: " .. tostring(sc) .. " ===")
    if type(functable) ~= "table" then
        errorLog("onCreateScript() AAAARRRGHHHH")
    else
        -- detour init() function to patch v to produce reasonable stackdumps,
        -- and optionally call the hook in _hackfixes.lua to inject another init() function
        local oldinit = functable.init
        if not oldinit then
            debugLog("OOPS: No init() present")
            oldinit = dummy
        end
        local function newinit(me)
            _detourInit()
            oldinit(me)
            if me and me ~= 0 then
                local hook = rawget(_G, "initHook")
                if hook then
                    return hook(sc, me)
                end
            end
        end
        functable.init = newinit
    end
        
    -- do the set, or it will crash
    rawset(tab, sc, functable)
end
setmetatable(_scriptfuncs, {
    __newindex = onCreateScript
})


-- detour global reads and writes to the currently active instance table
-- n = 0   will become   v.n = 0
-- local x = entity_x(n)   will become   local x = entity_x(v.n)
-- important here is that no instance-local variables accidentally become globals.
-- globals that get written into the instance table are no problem. (if it ever happens - doubt it)
local _G_meta =
{
    __index = function(tab, key)
        if INTERFACE_FUNCTIONS[key] then
            --debugLog("No interface function: " .. tostring(key))
            return nil
        end
        local v = rawget(_G, "v") -- force global lookup of v everytime
        if v then
            --debugLog("Lookup v[" ..  tostring(key) .. "] = " .. tostring(v[key]))
            return v[key]
        end
        --debugLog("Can't lookup v[" ..  tostring(key) .. "]")
    end,
    
    __newindex = function(tab, key, val)
        if key == "v" then -- v is set by the engine whenever script context is switched, this is no problem
            if val ~= nil and type(val) ~= "table" then
                errorLog_o("WARNING: _compat.lua: Setting v to non-table (" .. type(v) .. ")! This is BAD and WILL break something!\n" .. formatStack(2))
                -- but do it anyway.
            end
            rawset(tab, key, val)
            return
        end
        if INTERFACE_FUNCTIONS[key] then
            --debugLog("Setting interface: " .. tostring(key) .. " = " .. tostring(val))
            rawset(tab, key, val)
            return
        end
        if type(key) == "string" and looksLikeGlobal(key) then
            --debugLog("Setting global: " .. tostring(key) .. " = " .. tostring(val))
            rawset(_G, key, val)
            return
        end
        local v = rawget(_G, "v") -- force global lookup of v everytime
        if v then
            --debugLog("Setting v[" ..  tostring(key) .. "] = " .. tostring(val))
            v[key] = val
        else
            --debugLog("Can't set v[" .. tostring(key) .. "] = " .. tostring(val))
        end
    end
}
setmetatable(_G, _G_meta)


if rawget(_G, "FG_ENABLE_DEV_MODE") then
    local seen={}
    local function dump(t,i)
        seen[t]=true
        for k,x in pairs(t) do
            debugLog(i .. tostring(k) .. "  (" .. type(x) .. ") = " .. tostring(x))
            if type(x)=="table" and not seen[x] then
                dump(x,i.."  ")
            end
        end
    end
    debugLog("=== _compat.lua loaded -- _G table looks like this: ===")
    dump(_G, "")
    debugLog("============================")
end


rawset(_G, "__COMPAT_LOADED", true)
